/* -------------------------------------------------------------------	*/
/* Copyright (C) 2005 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.8								*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
#include <errors.h> 
#include <structur.h> 
#include <storage.h> 
#include <steen.h> 
#include <io.h> 

/* ---------------------------------------- fprint_magma_op   ---------- */

void fprint_magma_op(fp,Op) FILE *fp; Internal_Op Op;
{ register int i,max;
  for (max=N_ALG; Op[--max]==0 && (max >0); );
  fprintf(fp,"[%d",Op[0]);
  for (i=1; i<=max; i++) fprintf(fp,",%d",Op[i]);
  fprintf(fp,"]");
}


unsigned int xtrim[8] = {0xff, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe};

/* ---------------------- write_magma_TERM -------------------------------   */

write_magma_TERM(fp,x) FILE *fp;
                      TERM *x;
{ int j,length,s,n;
  char c;
            /* write_magma_TERM starts writing at the current position, */
	    /*  writes ops separated by commas, with a newline after   */
	    /*  each 10 ops                                            */

length = (x->len - 1) >> 3;
x->op[length++] &= xtrim[x->len &7];

	     {Internal_Op O;
              int seq=0;
	      int mid=0;
              int cnt=0;
	     
              for  (j=0,first_op(O,x->deg); j < length; j++) 
               { register unsigned mask = 0x80,
                                     xj = 255 & x->op[j];
				int   k;
                 for (k=0; (xj >0); k++, mask >>= 1)
                  if (xj & mask) { adv_op(O,k+8*j-seq);
				   if (mid) fprintf(fp,",");
				   else fprintf(fp,"[");
				   if (cnt >= 10)
				     { fprintf(fp,"\n   ");
				       cnt = 0;
				     }
                                   fprint_magma_op(fp,O);
				   mid = 1;
				   cnt += 1;
                                   seq = k+8*j;
                                   xj ^= mask;
                                 }
               }
             }


fprintf(fp,"]");
return(0);
}


/* ---------------------- write_magma_Element ------------------------------   */

write_magma_Element(fp,e) FILE *fp;
                         Element *e;
{ int g,i;

if (e == NULL) { fprintf(fp,"[]"); return(0); }

for (i=0, g=0; i < e->len; g++) 
{  
   if (g == 0) fprintf(fp,"[\n");
   if (g == e->item[i]->gen)
     write_magma_TERM(fp,e->item[i++]);
   else
     fprintf(fp,"[]");
   if (g == (e->item[e->len-1]->gen)) 
     fprintf(fp,"\n]");
   else
     fprintf(fp,",\n");
}
return(0);
}


/* ---------------------- main                    ------------------------   */



main(argc,argv) int argc; char *argv[];
{
Element *e;
int i,j,k,done,num;
int count;
int lim;
FILE *in,*out;


if (argc != 4) 
{ fprintf(stderr,
   "\nUsage: magmaout lim infile outfile ");
  fprintf(stderr,
   "\nwhere infile is a Diff.n file, outfile will hold the magma version, ");
  fprintf(stderr,
   "\nand lim is the maximum dimension wanted.\n\n");
    return(1);
  }


sscanf(argv[1],"%d",&lim);

if ((in = fopen(argv[2],"r")) == NULL) 
  { printf("\nError opening %s\n",argv[1]);
    return(1);
  }

if ((out = fopen(argv[3],"w")) == NULL) 
  { printf("\nError opening %s\n",argv[2]);
    return(1);
  }

fscanf(in,"%d %d",&i,&j);
fprintf(out,"deg := [");


done = 0;
count = 0;
while (!done) {

done = done || (EOF == fscanf(in,"%d",&j));
done = done || (j > lim);
if (!done) 
{
   if (count++ == 0) fprintf(out,"%d",j);
   else fprintf(out,", %d",j);

    skip_Element(in);
}
}

rewind(in);

fprintf(out,"],\n\nim := [\n\n");

fscanf(in,"%d %d",&i,&j);


for (i=0; i<count; ) 
{
   fscanf(in,"%d",&j);
   e = read_Element(in);
   write_magma_Element(out,e);
   if (++i == count) 
      fprintf(out,"];\n\n");
   else
      fprintf(out,",\n\n");
}

fclose(in);
fclose(out);
return(0);

}




