/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.6								*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>

#include <structur.h>
#include <io.h>
#include <storage.h>
#include <util.h>
#include <steen.h>

char Diff_hdr[] = "%10d %10d";   /* format of header in Diff files    */

typedef struct LIST { int item;
	       struct LIST *next; }  LIST;

int length(x) LIST *x;
{ int l;
  for (l=0; x != NULL; l++) x = x->next;
  return(l);
}

int twopow(x) int x;     /* return n if x is 2^n, else return -1 */
{ int result;
  if (x <= 0) return(-1);
  for (result=0; 0 == (1 & x); result++, x >>= 1);
  if (x == 1) return(result);
  else return(-1);
}


main(argc,argv) int argc; char *argv[];
{
int s,g,i,p,nread,dim,degree;
Element *e;
LIST *temp, *hi[N_ALG];
FILE *in, *out;
char filename[20];

if (argc != 3){ printf("\nUsage: summarize s outfile\n");
		exit(1);
              }

sscanf(argv[1],"%d",&s);

sprintf(filename,"Diff.%d",s);
if ((in = fopen(filename,"r")) == NULL) 
  { printf("\nError opening %s\n",filename);
    exit(1);
  }

if ((nread = fscanf(in,Diff_hdr,&dim,&degree)) != 2) 
{ printf("\nBad header in %s: only %d header items\n",filename,nread);
  exit(1);
}

printf("\nFiltration %d complete through degree %d.\n",s,degree);

if ((out = fopen(argv[2],"a")) == NULL) 
  { printf("\nError opening %s\n",argv[2]);
    exit(1);
  }

for (g=0; g<dim; g++)
{ fscanf(in,"%d",&degree);
  printf("%d_%d : %4d\n",s,g,degree);
  e = read_Element(in);
  for (i=0; i < N_ALG; i++) hi[i] = NULL;
  for (i=0; i<e->len; i++)
    { if ((e->item[i]->op[0] & 0x80) && ((p = twopow(e->item[i]->deg)) >=0)
          && (p < N_ALG))
      { temp = (LIST *) malloc(sizeof(LIST));
	temp->item = e->item[i]->gen;
	temp->next = hi[p];
	hi[p] = temp;
	printf("      h%d * %d\n",p,e->item[i]->gen);
      }
      if ((s > 0) && (e->item[i]->deg == 0))
	   printf("      1 * %d  Not minimal\n",e->item[i]->gen);
      if (e->item[i]->deg < 0) 
	   printf("      Error: negative degree %d",e->item[i]->deg);
    }
  for (i=0; i < N_ALG; i++) 
   if (hi[i] != NULL)
   { fprintf(out,"%d %d %d %d %d ",s,g,1,i,length(hi[i]));
     for (temp = hi[i]; temp != NULL; temp = temp->next)
       fprintf(out,"%d ",temp->item);
     fprintf(out,"\n");
   }
  return_Element(e);
}
}

