/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.6								*/
/* -------------------------------------------------------------------- */

/* These routine use but are not dependent upon the algebra routines.  */
/* Altering the routines in steen.h and steen.c are all that is needed */
/* to change algebras.                                                 */

#include <stdio.h>
/*#include <malloc.h> */
#include <stdlib.h>

#include <util.h>
#include <steen.h>
#include <structur.h>
#include <storage.h>

int mask[8] = {0xff,0x7f,0x3f,0x1f,0x0f,0x07,0x03,0x01};
static unsigned int trim[8] = {0xff, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe};

int leftbit[256] = {
 /* illegal    */    0,
 /* 0000 0001  */    7,
 /* 0000 001x  */    6,6,
 /* 0000 01xx  */    5,5,5,5,
 /* 0000 1xxx  */    4,4,4,4,4,4,4,4,
 /* 0001 xxxx  */    3,3,3,3,3,3,3,3,
                     3,3,3,3,3,3,3,3,
 /* 001x xxxx  */    2,2,2,2,2,2,2,2,  2,2,2,2,2,2,2,2,
                     2,2,2,2,2,2,2,2,  2,2,2,2,2,2,2,2,
 /* 01xx xxxx  */    1,1,1,1,1,1,1,1,  1,1,1,1,1,1,1,1,
                     1,1,1,1,1,1,1,1,  1,1,1,1,1,1,1,1,
                     1,1,1,1,1,1,1,1,  1,1,1,1,1,1,1,1,
                     1,1,1,1,1,1,1,1,  1,1,1,1,1,1,1,1,
 /* 1xxx xxxx  */    0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,
                     0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,
                     0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,
                     0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,
                     0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,
                     0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,
                     0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0,
                     0,0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0 };

/* ------------------------------------------------------------------ */
int first_op_in(x,Op) TERM *x; Internal_Op Op;
                         /* return the sequence number of the first op */
                         /* in x, and set Op to that operation         */
{
int i;

if (x == NULL) return(-1);
if (first_op(Op,x->deg) < 0) return(-1);
for (i=0; (i < (x->len-1) >> 3) && (x->op[i] == 0); i++);
if (x->op[i] == 0) return(-1);
  /* printf("\n1.Leftbit[%x]=%d  ",x->op[i],leftbit[x->op[i]]); */
i = (i << 3) + leftbit[x->op[i]] ;
adv_op(Op,i);
return(i);
}

/* ------------------------------------------------------------------ */
int next_op_in(x,Op,seqno) TERM *x; Internal_Op Op; int seqno;
                         /* return the sequence number of the next op 	*/
                         /* in x after Op, which has sequence number  	*/
                         /* seqno, and set Op to that operation       	*/
                         /* return -1 in no more ops                  	*/
			 /* Test for x == NULL, so that loops thru	*/
			 /* its ops can deallocate it when none remain	*/
{
int i,newseq,newop;

if (x == NULL) return(-1);
if ((newseq = seqno+1) >= x->len) return(-1);
i = newseq >>3;
newop = x->op[i]  & mask[newseq & 7];
if (newop)  {
  /* printf("\n2.Leftbit[%x]=%d  ",newop,leftbit[newop]);  */
   newseq = (i << 3) + leftbit[newop] ;
   adv_op(Op,newseq-seqno);
   return(newseq);
}

for (i++; i <= (x->len-1) >> 3; i++)
  if (x->op[i] != 0) {
     /* printf("\n3.Leftbit[%x]=%d  ",x->op[i],leftbit[x->op[i]]);   */
     newseq = (i << 3) + leftbit[x->op[i]] ;
     adv_op(Op, newseq - seqno);
     return(newseq);
  }
return(-1);

}

/* ----------------- first_seqno_in and next_seqno_in ---------------- */

int first_seqno_in(x) TERM *x;
                    /* return the sequence number of the first op in x */
                    /* return -1 if there is none                      */
{
int i;

if (x == NULL) return(-1);
if (dimA(x->deg) == 0) return(-1);
for (i=0; (i < (x->len-1) >> 3) && (x->op[i] == 0); i++);
if (x->op[i] == 0) return(-1);
return((i << 3) + leftbit[x->op[i]]) ;
}

/* ------------------------------------------------------------------ */

int next_seqno_in(x,seqno) TERM *x; int seqno;
                         /* return the sequence number of the next op 	*/
                         /* in x after op number seqno.               	*/
                         /* return -1 if no more ops                  	*/
			 /* Test for x == NULL, so that loops thru	*/
			 /* its ops can deallocate it when none remain	*/
{
int i,newseq,newop;

if (x == NULL) return(-1);
if ((newseq = seqno+1) >= x->len) return(-1);
i = newseq >>3;
newop = x->op[i]  & mask[newseq & 7];
if (newop)  {
  /* printf("\n2.Leftbit[%x]=%d  ",newop,leftbit[newop]);  */
   newseq = (i << 3) + leftbit[newop] ;
   return(newseq);
}

for (i++; i <= (x->len-1) >> 3; i++)
  if (x->op[i] != 0) {
     /* printf("\n3.Leftbit[%x]=%d  ",x->op[i],leftbit[x->op[i]]);   */
     newseq = (i << 3) + leftbit[x->op[i]] ;
     return(newseq);
  }
return(-1);

}
/* ------------------------------------------------------------------- */

int is_not_zero(x) TERM *x;
{
int i,length;

if (x == NULL) return(0);

length = (x->len - 1)/8;
x->op[length++] &= trim[x->len %8];

for (i=0; i<length; i++) if (x->op[i]) return(1);
return(0);
}

/* ----------------- condense ---------------------------------------  */

Element * condense(x,dim) TERM *x[]; int dim;
{
int i,j,k;
unsigned len;
Element *e;
TERM *t;

for (len=i=0; i<dim; i++) if (is_not_zero(x[i])) len++;

e = get_Element(len);

for (j=i=0; j<len; i++)
if (is_not_zero(x[i]))
{ e->item[j++] = t = get_TERM(x[i]->len);
  t->deg = x[i]->deg;
  t->gen = x[i]->gen;
  for (k=0; k <= (t->len - 1)>>3; k++) t->op[k] = x[i]->op[k];
}
return(e);
}

/* ------------- subtract ------------------------------------------   */

int subtract(x,y) TERM *x, *y;  /* replace x by x-y                   */
                                 /* Assume x and y share the same gen  */
                                 /* and deg, and hence len.            */
{ int i;

if (x==NULL) return(1);
if ((x->gen != y->gen) ||
    (x->len != y->len) ||
    (x->deg != y->deg)) return(1);

for (i=0; i <= (x->len-1) >> 3; i++) x->op[i] ^= y->op[i];
return(0);
}


/* ------------  copy_Element ------------------------------------- */

Element * copy_Element(y) Element *y; 
{
Element *x;
int i;

x = get_Element(y->len);
for (i=0; i < x->len; i++) 
	    x->item[i] = copy_TERM(y->item[i]);
return(x);
}


/* ------------  clean_copy ------------------------------------- */

Element * clean_copy(y, len) Element *y; int len;
{ /* copy the nonzero terms of y.  If len is > 0, it is the
    number of such.  Rteurn a pointer to a fresh Element which
    contains copies of these nonzero TERMs                      */
Element *x;
int ix,iy;

if (len <= 0) 
  for (iy=len=0; iy < y->len; iy++) if (is_not_zero(y->item[iy])) len++;
x = get_Element(len);
for (ix=iy=0; ix < x->len; iy++) 
	if (is_not_zero(y->item[iy])) 
	    x->item[ix++] = copy_TERM(y->item[iy]);
return(x);
}

/* ------------  clean_copy_exp ------------------------------------- */

Element * clean_copy_exp(y, len) TERM *y[]; int len;
{ /* copy the nonzero terms of the expanded Element y of length len.	*/
  /*   Rteurn a pointer to a fresh Element which contains copies of 	*/
  /*   these nonzero TERMs 						*/
Element *x;
int ix,iy,ln;

for (iy=ln=0; iy < len; iy++) if (is_not_zero(y[iy])) ln++;
x = get_Element(ln);
for (ix=iy=0; ix < x->len; iy++) 
	if (is_not_zero(y[iy])) 
	    x->item[ix++] = copy_TERM(y[iy]);
return(x);
}


/* ----------------- add  ------------------------------------  */

/* add the Elements *px and y point to, and place a pointer to 	*/
/*    the result in *px.  Cannibalize the Element **px to      	*/
/*    construct the sum, and return to the freelist any parts  	*/
/*    not used.  Do not alter *y.  *y may contain TERMs that   	*/
/*    are zero which should not be copied into the sum.        	*/
/* If the sum is zero, **px is deallocated and NULL is placed  	*/
/*    in *px.  						  	*/
/* If x == NULL, add functions like clean_copy			*/

void add (px,y) Element **px,*y;
{
Element * sum, *x;
int i,nonzero;
int ix,iy,is;            /* ix and iy are the next TERMs to add.  */
			 /* is is the spot to put it in.          */

x = *px;
if (x == NULL) *px = clean_copy(y,0);
else 
{
sum = get_Element(x->len + y->len);
ix = iy = is = 0;

while ((ix < x->len) || (iy < y->len))

  if ( ix == x->len) 
    { if (is_not_zero(y->item[iy] ))
	 sum->item[is++] = copy_TERM(y->item[iy++]);
      else iy++;
    }
  else if (( iy == y->len) || 
	   ( x->item[ix]->gen < y->item[iy]->gen))

	 sum->item[is++] = x->item[ix++];

  else if ( x->item[ix]->gen > y->item[iy]->gen)

    { if (is_not_zero(y->item[iy] ))
	 sum->item[is++] = copy_TERM(y->item[iy++]);
      else iy++;
    }
  else 
   { for (nonzero = i = 0; i <= (x->item[ix]->len -1)>>3; i++)
      if (x->item[ix]->op[i] ^= y->item[iy]->op[i]) nonzero = 1;
    if (nonzero) sum->item[is++] = x->item[ix];
      else return_TERM (x->item[ix]);
    ix++; iy++;
  }

return_Element(x);

if (is == 0) 
 { return_Element(sum);
   *px = NULL;
 }
else if (is < sum->len)
 { x = get_Element(is);
   for (i=0; i<is; i++) x->item[i] = sum->item[i];
   return_Element (sum);
   *px = x;
 }
else *px = sum;
}

return;
}

/* ----------------- add_exp  --------------------------------  */

/* Same as add, except that the second arg is an expanded	*/
/*   Element of length the 3rd arg.				*/

void add_exp (px,y,len) Element **px; TERM **y; int len;
{
Element * sum, *x;
int i,nonzero;
int ix,iy,is;            /* ix and iy are the next TERMs to add.  */
			 /* is is the spot to put it in.          */

x = *px;
if (x == NULL) *px = clean_copy_exp(y,len);
else 
{
sum = get_Element(x->len + len);
ix = iy = is = 0;

while ((ix < x->len) || (iy < len))

  if ( ix == x->len) 
    { if (is_not_zero(y[iy] ))
	 sum->item[is++] = copy_TERM(y[iy++]);
      else iy++;
    }
  else if (( iy == len) || 
	   ( x->item[ix]->gen < iy))

	 sum->item[is++] = x->item[ix++];

  else if ( x->item[ix]->gen > iy)

    { if (is_not_zero(y[iy] ))
	 sum->item[is++] = copy_TERM(y[iy++]);
      else iy++;
    }
  else if (y[iy] == NULL)
   { sum->item[is++] = x->item[ix++];
     iy++;
   }
  else
   { for (nonzero = i = 0; i <= (x->item[ix]->len -1)>>3; i++)
      if (x->item[ix]->op[i] ^= y[iy]->op[i]) nonzero = 1;
    if (nonzero) sum->item[is++] = x->item[ix];
      else return_TERM (x->item[ix]);
    ix++; iy++;
  }

return_Element(x);

if (is == 0) 
 { return_Element(sum);
   *px = NULL;
 }
else if (is < sum->len)
 { x = get_Element(is);
   for (i=0; i<is; i++) x->item[i] = sum->item[i];
   return_Element (sum);
   *px = x;
 }
else *px = sum;
}

return;
}

/* ---------------------- copy_TERM --------------------------- */

TERM * copy_TERM(x) TERM *x;
{ int i;
  TERM *y = get_TERM(x->len);

  y->gen = x->gen;
  y->deg = x->deg;
  for (i=0; i <= (x->len-1) >> 3; i++) y->op[i] = x->op[i];
  return(y);
}



/* ----------------------------------------------------------- */
