/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.5								*/
/* -------------------------------------------------------------------- */

/* Designed to work on modules over A(2), not all of A                  */
/* Done ad hoc, so will not generalize nicely to A(n) or other          */
/* subalgebras.  Clearly someone should write routines which work       */
/* from a profile function for a subalgebra and are therefore           */
/* perfectly general.                                                   */

/* Includes a big ad hoc piece of code called expansions.c               */
/*  (see initialize())                                                  */

#include <stdio.h>
#include <string.h>
/* #include <malloc.h> */
#include <stdlib.h>
#include <util.h>
#include <steen.h>
#include <structur.h>
#include <storage.h>
#include <io.h>
#include <errors.h>

typedef struct Steen_Op {
                    int first_sq;
        struct Steen_Op *first_rest;
        struct Steen_Op *other_terms; } Steen_Op;

typedef struct module_element {
                               int first;
             struct module_element *next;} module_element;

module_element ***sq_value=NULL;
int numgen, *degrees;
int initialized=0;

Steen_Op *Expansion[8][4][2];

/* Expansion[i][j][k] tells how to write Sq(i,j,k) in terms of       */
/* Sq^1, ... Sq^7.                                                    */
/* initialize() fills it.                                             */

/* ------------------------------------------------------------------ */
module_element * copy_module_element(x) module_element *x;
{ module_element *z;

if (x == NULL) return(NULL);

z = (module_element *) malloc(sizeof(module_element));
z->first = x->first;
z->next = copy_module_element(x->next);
return(z);
}

/* ------------------------------------------------------------------ */
module_element * insert_gen(g,e)  int g; module_element *e;
{
  if ((e==NULL) || g < e->first)
  {
    module_element *e1=(module_element *)malloc(sizeof(module_element));

    e1->first=g;
    e1->next=e;
    return(e1);
  }
  if (g == e->first)  /*bad data file */
  {
    fprintf(stderr,"bad data file: Def\n");
    return (e->next);
  }
  e->next=insert_gen(g,e->next);
  return(e);
}

/* ------------------------------------------------------------------ */
Steen_Op *create(n,e) int n; int *e;
{
  Steen_Op *v;

if (n==0) return(NULL);
v = (Steen_Op *)malloc(sizeof(Steen_Op));
v->first_sq = e[0];
v->other_terms = NULL;
v->first_rest = create(n-1,e+1);
return(v);
}


/* ------------------------------------------------------------------ */
TERM * condense_Op(v) Steen_Op *v;
{
  TERM *g,*t;
  int s,i,seqno;
  Internal_Op Op, Op2;

if (v == NULL)     /* return the identity operation */
{
  t = get_TERM(1);
  t->gen = 0;
  t->deg = 0;
  t->op[0] = 0x80;
  return(t);
}

t = condense_Op(v->first_rest);
/* Let g = Sq(i) * t              */

i = v->first_sq;
Op[0]=i;
Op[1]=Op[2]=0;

g = get_TERM(dimA(i+t->deg));
g->gen = 0; 
g->deg = i+t->deg;
zero_op(g);

     for (seqno = first_op_in(t,Op2);
            seqno >= 0;
               seqno = next_op_in(t,Op2,seqno))
      mult(Op,Op2,g);

/* Now add the rest to g    */

v = v->other_terms;
if ( v != NULL)
  if (subtract(g,condense_Op(v)))
     { fprintf(stderr,"\nMismatched TERMs:\n");
       write_TERM(stderr,g,'i');
       fprintf(stderr,"\nand\n");
       write_TERM(stderr,condense_Op(v),'i');
       exit(1);
     }

return(g);

}

/* ------------------------------------------------------------  */
int count_Steen_Op(v) Steen_Op *v;
{
if (v==NULL) return(0);

return(1+count_Steen_Op(v->first_rest)+count_Steen_Op(v->other_terms));

}


/* ------------------------------------------------------------  */
void print_Steen_Op(prefix,v) char *prefix; Steen_Op *v;
{
char *preL, *preR;
if (v==NULL) return;

printf("%s%d\n",prefix,v->first_sq);
preR = (char *)malloc((2+strlen(prefix))*sizeof(char));
strcpy(preR,prefix);
strcat(preR,"R");
print_Steen_Op(preR,v->first_rest);
preL = (char *)malloc((2+strlen(prefix))*sizeof(char));
strcpy(preL,prefix);
strcat(preL,"L");
print_Steen_Op(preL,v->other_terms);

}

/* ------------------------------------------------------------------ */
void initialize()
{ FILE *inputfile;
  int i, j, k, gen, op, count, g;
  int e[10];
  Steen_Op *v;

  if (NULL == (inputfile=fopen("Def","r")))
    terminate("\nCannot open module definition: Def\n"
              ,BAD_FILE);

  fscanf(inputfile, "%d", &numgen);

  degrees=(int *)malloc(numgen * sizeof(int));
  for (gen=0;gen<numgen;gen++)
    fscanf(inputfile, "%d ",&degrees[gen]);

  sq_value=(module_element ***)malloc(numgen*sizeof(module_element **));
  for (gen=0;gen<numgen;gen++)
  {
    sq_value[gen]=(module_element **)malloc(
         (1+degrees[numgen-1] - degrees[gen])*sizeof(module_element *));
    sq_value[gen][0] = (module_element *)malloc(sizeof(module_element));
    sq_value[gen][0]->first = gen;
    sq_value[gen][0]->next = NULL;
    for(i=1;i<=degrees[numgen-1] - degrees[gen];i++)
      sq_value[gen][i]=NULL;
  }
                /* sq_value[gen][op]'s are initially NULL */

  while (EOF != fscanf(inputfile, "%d %d %d", &gen,&op,&count))
    for(i=0;i<count;i++)
    { fscanf(inputfile, "%d", &g);
      sq_value[gen][op]=insert_gen(g,sq_value[gen][op]);
    }
fclose(inputfile);

/* Now set up expansions of elements of A(2)                       */

for (i=0; i<8; i++)
for (j=0; j<4; j++)
for (k=0; k<2; k++)
  Expansion[i][j][k] = NULL;


for (i=0; i<8; i++)
{
  Expansion[i][0][0] = (Steen_Op *)malloc(sizeof(Steen_Op));
  Expansion[i][0][0]->first_sq = i;
  Expansion[i][0][0]->first_rest = NULL;
  Expansion[i][0][0]->other_terms = NULL;
}

/* Now go get the details.  */

#include<expansions.c>

initialized=1;
return;
}

/* ------------------------------------------------------------------ */
int mod_basis(t,adegree) int t, **adegree;
{ int i,count;

if (!initialized) initialize();

for (count = i = 0; i < numgen; i++)
  if (degrees[i] <= t) count++;

*adegree = degrees;
return(count);
}

/* ------------------------------------------------------------------ */
void print_module_element(e)  module_element *e;
{
  module_element *i=NULL;

  if (e != NULL)
    for(i=e;i != NULL;i=i->next)
      printf("%d + ",i->first);
  printf("done\n");
  return;
}

/* --------------------------------------------------------------- */
module_element * modadd(x,y) module_element *x,*y;
{
  module_element *x1=NULL,*y1=NULL;
                                   /* destroys x and y as it adds */
  if (x==NULL) return(y);
  if (y==NULL) return(x);
  if (x->first < y->first)
  {
    x->next = modadd(x->next,y);
    return(x);
  }
  if (x->first > y->first)
  {
    y->next = modadd(x,y->next);
    return(y);
  }
  /* first terms are equal, hence cancel */
  x1=x->next;
  y1=y->next;
  free(x);
  free(y);
  return(modadd(x1,y1));
}

/* --------------------------------------------------------------- */
module_element * actone(op,x) int op; module_element *x;
{
  module_element *e=NULL,*f=NULL;
  char errmess[100];

  if (x==NULL)
    return(x);
  if ((x->first >= numgen) || (x->first < 0))
  { sprintf(errmess,"\nBad gen %d out of range %d to %d in actone\n",
            x->first,0,numgen-1);
    terminate(errmess,MODERR);
  }
  if (op + degrees[x->first] > degrees[numgen-1])
    e=NULL;
  else
    e = copy_module_element(sq_value[x->first][op]);
  f=actone(op,x->next);  /* if op is acting on an element x which is the
                            sum of more than one element, say
                            x1 + x2 + ... + xn, then f will equal op
                            acting on x2 + ... + xn                       */
  return(modadd(e,f));
}

/* ------------------------------------------------------------------ */
module_element * act(tree,x) Steen_Op *tree; module_element *x;
{
  module_element *e=NULL,*f=NULL;
  char errmess[100];

  if ((x->first >= numgen) || (x->first < 0))
  { sprintf(errmess,"\nBad gen %d out of range %d to %d in act\n",
            x->first,0,numgen-1);
    terminate(errmess,MODERR);
  }
  if (tree == NULL)
    return(x);
  if ((tree->first_rest) == NULL)   /* tree->first_sq in modadd statement
                                       below will act on x                */
    e=x;
  else
    e=act(tree->first_rest,x);
  if ((tree->other_terms) == NULL)  /* add NULL to modadd statement below */
    f=NULL;
  else
    f=act(tree->other_terms,x);
  return(modadd(actone(tree->first_sq,e),f));
}

/* ------------------------------------------------------------------ */
void modact(dim,dx,t,Op,opseqno,dg,g_degree)
       int dim,t,opseqno,g_degree; TERM **dx, **dg; Internal_Op Op;
{
int g;
module_element *value, *newvalue;

if (!initialized) initialize();

for (value=NULL,g=dim; g-- > 0;)
  if ((dg[g] != NULL) && is_not_zero(dg[g]))
    value = insert_gen(g,value);

newvalue = act(Expansion[Op[0]][Op[1]][Op[2]],value);

for (g=0; g < dim; g++)
if ((newvalue == NULL) || (g < newvalue->first))
{    if (dx[g] != NULL) zero_op(dx[g])
}
else { dx[g]->op[0] = 0x80;
       newvalue = newvalue->next;
     }
return;
}

/* -------------------------------------------------------------------------- */
