/* -------------------------------------------------------------------	*/
/* Copyright (C) 2000 Robert R. Bruner.  All rights reserved.		*/
/* Contact the author at						*/
/*                    rrb@math.wayne.edu				*/
/*                    Dept of Mathematics				*/
/*                    Wayne State University				*/
/*                    Detroit, Michigan  48202				*/
/*                    USA						*/
/*									*/
/* Version 1.6								*/
/* -------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>

#include <errors.h>
#include <tensor.h>
#include <setup.h>

main(argc,argv) int argc; char *argv[];

{
int numstems=PAGEWIDTH/BOXSIZE;
int s_lo, st_lo, **Num, s,g,minstem,maxstem, i, stem, stem1, last,
    ds,defaultstyle, **Styles, s0, g0, count, g1, n;

Tensor_shape *omit, *posn, *shape;
FILE *Omit, *Sty, *Out, *Lines;

char message[100];

/* -------------- check calling sequence -----------------------------  */
if ((argc != 7) && (argc != 8))
{
  printf("\nUsage: olddisplay s_lo st_lo lines omit out label [style]]\n");
  printf("    creates a ps file with lower left corner at\n");
  printf("    filt s_lo, stem st_lo, using data from lines\n");
  printf("    leaving out dots listed in omit and any lines\n");
  printf("    involving them, putting the result into out, \n");
  printf("    using line styles from style if present.     \n");
  printf("    Omit format: g1 s1 g2 s2 ...     \n");
  printf("    Chart is labelled by the string label.\n\n");
  exit(1);
}

/* ------------------ get s_lo and st_lo --------------------------------- */

sscanf(argv[1],"%d",&s_lo);
sscanf(argv[2],"%d",&st_lo);
last = st_lo + numstems;


/* ------------------ position dots  ----------------------------------- */

shape = read_shape(-1);
posn = read_shape(-1);
omit = read_shape(-1);

maxstem = minstem = shape->deg[0][0];
for (s=0; s<=shape->s; s++)
 for (g=0; g<shape->numgen[s]; g++)
 { if (maxstem < shape->deg[s][g]-s) maxstem = shape->deg[s][g]-s;
   if (minstem > shape->deg[s][g]-s) minstem = shape->deg[s][g]-s;
 }

Num = (int **) malloc(((++maxstem)-minstem)*sizeof(int *));
Num -= minstem;
		/*      Now Num[minstem] through Num[maxstem-1] exist	*/
for (n=minstem; n < maxstem; n++)
{  Num[n] = (int *) malloc((1+shape->s)*sizeof(int));
   for (s=0; s<= shape->s; s++) Num[n][s] = 0;
}

for (s=0; s<=shape->s; s++)
 for (g=0; g<shape->numgen[s]; g++)
 { stem = shape->deg[s][g]-s;
   posn->deg[s][g] = Num[stem][s]++;
   omit->deg[s][g] = 0;
 }

if ((Omit = fopen(argv[4],"r")) == NULL)
{  sprintf(message,"\nCannot open file: %s\n",argv[4]);
   terminate(message,BAD_WORKFILE);
}
while (EOF != fscanf(Omit,"%d %d",&g,&s))
     omit->deg[s][g] = 1;
fclose(Omit);

defaultstyle = 0;
Styles = (int **) malloc((2+shape->s)*sizeof(int *));
Styles++;
Styles[-1] = (int *) malloc(5*sizeof(int));
for (g=0; g<5; g++) Styles[-1][g] = defaultstyle;
for (s=0; s <= shape->s; s++)
{  Styles[s] = (int *) malloc(shape->numgen[s]*sizeof(int));
   for (g=0; g< shape->numgen[s]; g++) Styles[s][g] = defaultstyle;
}
if (argc == 7) 
{ Styles[1][0] = 1;
  Styles[1][1] = 1;
  Styles[1][2] = 2;
}

if (argc == 8)
{  if ((Sty = fopen(argv[7],"r")) == NULL)
   {  sprintf(message,"\nCannot open file: %s\n",argv[7]);
      terminate(message,BAD_WORKFILE);
   }
   while (EOF != fscanf(Sty,"%d %d %d",&s,&g,&i))
     Styles[s][g] = i;
}

/* ------------------ Now draw dots  ----------------------------------- */

if ((Out = fopen(argv[5],"w")) == NULL)
{  sprintf(message,"\nCannot open file: %s\n",argv[5]);
   terminate(message,BAD_WORKFILE);
}

preface(Out);
newpage(Out,s_lo,st_lo,argv[6]);

if (last >= maxstem) last = maxstem-1;
else
{  printf("\n %d pages remain. Run display %d %d, display %d %d, ...\n",
	  1+(--maxstem - last - 1)/numstems,s_lo,last,s_lo,last+numstems);
}
for (s=s_lo; s<=shape->s; s++)
for (g=0; g<shape->numgen[s]; g++)
 { stem = shape->deg[s][g]-s;
   if ((omit->deg[s][g] == 0) 
       && (stem >= minstem)
       && (stem >= st_lo)
       && (stem <= last))
       makedot(Out,s-s_lo,stem-st_lo,posn->deg[s][g]);
 }


/* ------------------ Now draw lines  ----------------------------------- */


if ((Lines = fopen(argv[3],"r")) == NULL)
{  sprintf(message,"\nCannot open file: %s\n",argv[3]);
   terminate(message,BAD_WORKFILE);
}

while (EOF != fscanf(Lines,"%d %d %d %d %d",&s,&g,&s0,&g0,&count))
{ if (s0 <0) ds = 4*g0;
   else ds = s0;
  for (i=0; i<count; i++)
  { fscanf(Lines,"%d",&g1);
    if ((s > 0)      /* Ignore hom deg 0 "multiples" (ficticious)	*/
	&& !((omit->deg[s][g]) | (omit->deg[s-ds][g1])))
    { stem = shape->deg[s][g] - s;
      stem1 = shape->deg[s-ds][g1] - s + ds; 
      if (((stem >= st_lo) && (stem <= last)) ||
          ((stem1 >= st_lo) && (stem1 <= last)))
      cnnct(Out,s-s_lo,stem-st_lo,posn->deg[s][g],
	          s-s_lo-ds,stem1-st_lo,posn->deg[s-ds][g1],Styles[s0][g0]);
    }
  }
}
endpage(Out);
fclose(Out);
fclose(Lines);
   
}

